import 'reflect-metadata'
import {Service} from "typedi"
import {db} from "../includes"
import {fillIExercise, fillITextbook, fillIToc} from "../entity-fill"
import {text} from "express"

@Service()
export class TextbookModel {
    existTextbook(textbook: string) {
        let raw = db.prepare('select rowid from solutions where textbook=? limit 1').get(textbook)
        return !!raw?.rowid
    }

    existSection(textbook: string, section: string) {
        let raw = db.prepare('select rowid from solutions where textbook=? and section=? limit 1').get(textbook, section)
        return !!raw?.rowid
    }

    existExercise(textbook: string, section: string, exercise: string) {
        let raw = db.prepare('select rowid from solutions ' +
            'where textbook=? and section=? and exercise=? limit 1').get(textbook, section, exercise)
        return !!raw?.rowid
    }

    getTextbooks() {
        let raw = db.prepare('SELECT textbook as title,count(rowid) as quantity from solutions GROUP BY textbook').all()
        return raw.map(value => fillITextbook(value))
    }

    findExercisesByTextbookSection(textbook: string, section: string) {
        let raw = db.prepare('SELECT exercise,section,part,page,html ' +
            'from solutions ' +
            'where textbook=? ' +
            'and section=? order by exercise asc').all(textbook, section)
        return raw.map(value => fillIExercise(value))
    }

    findExercise(textbook: string, section: string, exercise: string) {
        let raw = db.prepare('SELECT exercise,section,part,page,html ' +
            'from solutions ' +
            'where textbook=? ' +
            'and section=? and exercise=?').get(textbook, section, exercise)
        return fillIExercise(raw)
    }

    findExercisesByTextbookPage(textbook: string, page: number) {
        let raw = db.prepare('SELECT exercise,section,part,page,html ' +
            'from solutions ' +
            'where textbook=? ' +
            'and page=? order by exercise asc').all(textbook, page)
        return raw.map(value => fillIExercise(value))
    }

    findTocByTextbook(textbook: string) {
        let raw = db.prepare('SELECT section, ' +
            'count(rowid) as count, ' +
            'min(page) as pageBegin, ' +
            'max(page) as pageEnd ' +
            'FROM "solutions" ' +
            'where textbook=? ' +
            'group by section ' +
            'order by pageBegin asc').all(textbook)
        return raw.map(value => fillIToc(value))
    }
}
